@@ -1,7 +1,10 @@
-/* Conky, a system monitor, based on torsmo
+/* -*- mode: c++; c-basic-offset: 4; tab-width: 4; indent-tabs-mode: t -*-
+ * vim: ts=4 sw=4 noet ai cindent syntax=cpp
+ *
+ * Conky, a system monitor, based on torsmo
  *
  * Copyright (c) 2009 Toni Spets
- * Copyright (c) 2005-2009 Brenden Matthews, Philip Kovacs, et. al.
+ * Copyright (c) 2005-2012 Brenden Matthews, Philip Kovacs, et. al.
  *	(see AUTHORS)
  * All rights reserved.
  *
@@ -22,41 +25,41 @@
 #ifndef LUA_H_
 #define LUA_H_
 
+extern "C" {
 #include <lua.h>
 #include <lauxlib.h>
 #include <lualib.h>
+}
 
-#include "config.h"
+#include <config.h>
 
-#ifdef X11
+#ifdef BUILD_X11
 #include "x11.h"
-#endif /* X11 */
+#endif /* BUILD_X11 */
 
 #define LUAPREFIX "conky_"
 
-/* load a lua script */
-void llua_load(const char *script);
-/* call a function with args, and return a string from it (must be free'd) */
-char *llua_getstring(const char *args);
-/* call a function with args, and return a string from it (must be free'd) */
-char *llua_getstring_read(const char *function, const char *arg);
-/* call a function with args, and put the result in ret */
-int llua_getnumber(const char *args, double *ret);
-/* close lua stuff */
-void llua_close(void);
 #ifdef HAVE_SYS_INOTIFY_H
 /* check our lua inotify status */
 void llua_inotify_query(int wd, int mask);
 #endif /* HAVE_SYS_INOTIFY_H */
 
-#ifdef X11
+void llua_startup_hook(void);
+void llua_shutdown_hook(void);
+
+#ifdef BUILD_X11
 void llua_draw_pre_hook(void);
 void llua_draw_post_hook(void);
-void llua_set_draw_pre_hook(const char *args);
-void llua_set_draw_post_hook(const char *args);
 
 void llua_setup_window_table(int text_start_x, int text_start_y, int text_width, int text_height);
 void llua_update_window_table(int text_start_x, int text_start_y, int text_width, int text_height);
-#endif /* X11 */
+#endif /* BUILD_X11 */
+
+void llua_setup_info(struct information *i, double u_interval);
+void llua_update_info(struct information *i, double u_interval);
+
+void print_lua(struct text_object *, char *, int);
+void print_lua_parse(struct text_object *, char *, int);
+double lua_barval(struct text_object *);
 
 #endif /* LUA_H_*/
